#ifndef __BOOST__IO__FORMATTER__HPP
#define __BOOST__IO__FORMATTER__HPP
   namespace boost { namespace io
   {
      template< typename FormatType, class ReferenceType >
      class formatter_t
      {
         public:
            typedef FormatType                             listfmt_type;
            typedef format_traits< listfmt_type >          traits_type;
         private: // format control
            listfmt_type               __openlist;
            listfmt_type               __closelist;
            listfmt_type               __separator;
         public:
            inline listfmt_type                  openlist()  const throw();
            inline listfmt_type                  closelist() const throw();
            inline listfmt_type                  separator() const throw();
         public:
            inline ReferenceType &               format
                                                 (
                                                    listfmt_type, // open  list
                                                    listfmt_type, // close list
                                                    listfmt_type  // seperator
                                                 ) throw();
            inline ReferenceType &               format
                                                 (
                                                    listfmt_type, // open  list
                                                    listfmt_type  // close list
                                                 ) throw();
            inline ReferenceType &               format
                                                 (
                                                    listfmt_type  // seperator
                                                 ) throw();
         private: // spacing control
            bool                       __spaceBeforeSep;
            bool                       __spaceAfterSep;
            bool                       __spaceAroundList;
         public:
            inline bool                          spaceBeforeSep()  const throw();
            inline bool                          spaceAfterSep()   const throw();
            inline bool                          spaceAroundList() const throw();
         public:
            inline ReferenceType &               space
                                                 (
                                                    bool, // before separator
                                                    bool, // after  separator
                                                    bool  // around list
                                                 ) throw();
            inline ReferenceType &               space
                                                 (
                                                    bool, // before separator
                                                    bool  // after  separator
                                                 ) throw();
            inline ReferenceType &               space
                                                 (
                                                    bool  // around list
                                                 ) throw();
         public: // configure from a formatter object
            template< typename RT2 >
            inline ReferenceType &               format
                                                 (
                                                    const formatter_t< FormatType, RT2 > & const_t fmt
                                                 )
            {
               return( format( fmt.openlist(), fmt.closelist(), fmt.separator()));
            }
            template< typename RT2 >
            inline ReferenceType &               space
                                                 (
                                                    const formatter_t< FormatType, RT2 > & const_t fmt
                                                 )
            {
               return( space( fmt.spaceBeforeSep(), fmt.spaceAfterSep(), fmt.spaceAroundList()));
            }
            template< typename RT2 >
            inline ReferenceType &               inherit
                                                 (
                                                    const formatter_t< FormatType, RT2 > & const_t fmt
                                                 )
            {
               format( fmt.openlist(), fmt.closelist(), fmt.separator());
               return( space( fmt.spaceBeforeSep(), fmt.spaceAfterSep(), fmt.spaceAroundList()));
            }
         public: // constructors
            inline           formatter_t() throw();
            template< typename RT2 >
            inline           formatter_t( const formatter_t< FormatType, RT2 > & const_t fmt ) throw():
               __openlist(        fmt.openlist()),
               __closelist(       fmt.closelist()),
               __separator(       fmt.separator()),
               __spaceBeforeSep(  fmt.spaceBeforeSep()),
               __spaceAfterSep(   fmt.spaceAfterSep()),
               __spaceAroundList( fmt.spaceAroundList())
            {
            }
      };

      template< typename FormatType >
      class formatter: public formatter_t< FormatType, formatter< FormatType > >
      {
         public:
            inline           formatter() throw();
            template< typename RT2 >
            inline           formatter( const formatter_t< FormatType, RT2 > & const_t fmt ) throw():
               formatter_t< FormatType, formatter >( fmt )
            {
            }
      };
   }}

#  include "formatter.ipp"
#endif
